﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using WP8Sample.Resources;

using UrbanAirship;
using UrbanAirship.Push;

namespace WP8Sample
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();

            // Sample code to localize the ApplicationBar
            //BuildLocalizedApplicationBar();
        }

        // Sample code for building a localized ApplicationBar
        //private void BuildLocalizedApplicationBar()
        //{
        //    // Set the page's ApplicationBar to a new instance of ApplicationBar.
        //    ApplicationBar = new ApplicationBar();

        //    // Create a new button and set the text value to the localized string from AppResources.
        //    ApplicationBarIconButton appBarButton = new ApplicationBarIconButton(new Uri("/Assets/AppBar/appbar.add.rest.png", UriKind.Relative));
        //    appBarButton.Text = AppResources.AppBarButtonText;
        //    ApplicationBar.Buttons.Add(appBarButton);

        //    // Create a new menu item with the localized string from AppResources.
        //    ApplicationBarMenuItem appBarMenuItem = new ApplicationBarMenuItem(AppResources.AppBarMenuItemText);
        //    ApplicationBar.MenuItems.Add(appBarMenuItem);
        //}

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            Logger.Debug("On Navigated To: " + e.Uri.ToString());

            if (PushManager.Shared.ToastOptInStatus == OptInStatus.NotPrompted)
            {
                MessageBoxResult result = MessageBox.Show(
                    "This application would like to send you push notifications. Press OK to opt-in.", "Push Notifications",
                    MessageBoxButton.OKCancel);
                if (result == MessageBoxResult.OK)
                {
                    Logger.Debug("Opting In.");
                    PushManager.Shared.EnabledNotificationTypes = NotificationType.Toast | NotificationType.Tile;
                    PushManager.Shared.ToastOptInStatus = OptInStatus.OptIn;
                }
                else
                {
                    Logger.Debug("Opting Out.");
                    PushManager.Shared.ToastOptInStatus = OptInStatus.OptOut;
                }
            }

            updateScreen();

            PushManager.Shared.RegistrationEvent += (sender, eventArgs) =>
            {
                updateScreen();
            };

            base.OnNavigatedTo(e);
        }

        private void updateScreen()
        {
            this.ToastCheckBox.IsChecked = PushManager.Shared.ToastEnabled;
            this.TileCheckBox.IsChecked = PushManager.Shared.TilesEnabled;
            this.txtLibraryVersion.Text = UAirship.Version;
            this.txtAlias.Text = PushManager.Shared.Alias;
            this.txtTags.Text = string.Join(",", PushManager.Shared.Tags);
            this.chkCycleTile.IsChecked = FindTile(CYCLE_TILE_URI) != null;
            this.chkFlipTile.IsChecked = FindTile(FLIP_TILE_URI) != null;
        }

        private void ToastNotificationsEnabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.ToastEnabled = true;
        }

        private void ToastNotificationsDisabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.ToastEnabled = false;
        }

        private void TileNotificationsEnabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.TilesEnabled = true;
        }

        private void TileNotificationsDisabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.TilesEnabled = false;
        }

        private void AliasLostFocus(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.Alias = txtAlias.Text;
            PushManager.Shared.UpdateRegistration();
        }

        private void TagsLostFocus(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.Tags = (from t in txtTags.Text.Split(',') select t.Trim()).ToArray();
            PushManager.Shared.UpdateRegistration();
        }

        private static readonly string CYCLE_TILE_URI = "/MainPage.xaml?source=CycleTile";
        private static readonly string FLIP_TILE_URI = "/MainPage.xaml?source=FlipTile";

        private ShellTile FindTile(string partOfUri)
        {
            ShellTile shellTile = ShellTile.ActiveTiles.FirstOrDefault(
                tile => tile.NavigationUri.ToString().Contains(partOfUri));
            return shellTile;
        }

        private void FlipTileChecked(object sender, RoutedEventArgs e)
        {
            ShellTile tile = FindTile(FLIP_TILE_URI);
            if (tile == null)
            {
                FlipTileData data = new FlipTileData()
                {
                    Title = "UADemo Flip Tile",
                    BackTitle = "UADemo Back Title",
                    BackContent = "This side of the tile shows at random intervals.",
                    WideBackContent = "This content shows on the back of wide tiles only."
                };
                ShellTile.Create(new Uri(FLIP_TILE_URI, UriKind.Relative), data, supportsWideTile: true);
                Logger.Debug("Created flip tile.");
            }
        }

        private void FlipTileUnchecked(object sender, RoutedEventArgs e)
        {
            ShellTile tile = FindTile(FLIP_TILE_URI);
            if (tile != null)
            {
                tile.Delete();
                Logger.Debug("Deleted flip tile.");
            }
        }

        private void CycleTileChecked(object sender, RoutedEventArgs e)
        {
            ShellTile tile = FindTile(CYCLE_TILE_URI);
            if (tile == null)
            {
                CycleTileData data = new CycleTileData()
                {
                    Count = 10,
                    Title = "UADemo Cycle Tile"
                };
                ShellTile.Create(new Uri(CYCLE_TILE_URI, UriKind.Relative), data, supportsWideTile: true);
                Logger.Debug("Created cycle tile");
            }
        }

        private void CycleTileUnchecked(object sender, RoutedEventArgs e)
        {
            ShellTile tile = FindTile(CYCLE_TILE_URI);
            if (tile != null)
            {
                tile.Delete();
                Logger.Debug("Deleted cycle tile.");
            }
        }
    }
}