﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Data.Xml.Dom;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using UrbanAirship;
using UrbanAirship.Push;

// The Blank Application template is documented at http://go.microsoft.com/fwlink/?LinkId=234227

namespace Win8Sample
{
    /// <summary>
    /// Provides application-specific behavior to supplement the default Application class.
    /// </summary>
    sealed partial class App : Application
    {
        //example of a custom registration listener
        //this can be customized by the app developer
        private class RegistrationEventListener
        {
            //constructor, implicitly subscribes to the event
            public RegistrationEventListener()
            {
                PushManager.Shared.RegistrationEvent += RegistrationComplete;
            }

            //unsubscribes from the event
            public void Detach()
            {
                PushManager.Shared.RegistrationEvent -= RegistrationComplete;
            }

            //called when the event fires
            private void RegistrationComplete(object sender, RegistrationEventArgs e)
            {
                Logger.Info("Registration complete, apid: " + e.Apid + " valid: " + e.IsValid);
            }
        }

        //example of a custom push received listener
        //this can be customized by the app developer
        private class PushReceivedEventListener
        {
            //constructor, implicitly subscribes to the event
            public PushReceivedEventListener()
            {
                PushManager.Shared.PushReceivedEvent += PushReceived;
            }

            //unsubscribes from the event
            public void Detach()
            {
                PushManager.Shared.PushReceivedEvent -= PushReceived;
            }

            //called when the event fires
            private void PushReceived(object sender, PushReceivedEventArgs e)
            {
                Logger.Info("Push received!");

                UrbanAirship.Push.PushNotification notification = e.Notification;

                if (notification.Type == UrbanAirship.Push.NotificationType.Toast)
                {
                    ToastNotificationData data = notification.ToastData;

                    Logger.Info("Text: ");
                    foreach (var item in data.Text)
                    {
                        Logger.Info(item);
                    }

                    //if needed, the full XML payload is included for further inspection
                    XmlDocument payload = (XmlDocument)data.Payload;
                    Logger.Info("Full XML payload:");
                    Logger.Info(payload.GetXml());
                }
                else if (notification.Type == UrbanAirship.Push.NotificationType.Tile)
                {
                    TileNotificationData data = notification.TileData;
                    //if needed, the full XML payload is included for further inspection
                    XmlDocument payload = (XmlDocument)data.Payload;
                    Logger.Info("Full XML payload:");
                    Logger.Info(payload.GetXml());
                }
            }
        }

        //example of a custom push activated listener
        //this can be customized by the app developer
        private class PushActivatedEventListener
        {
            //constructor, implicitly subscribes to the event
            public PushActivatedEventListener()
            {
                PushManager.Shared.PushActivatedEvent += PushActivated;
            }

            //unsubscribes from the event
            public void Detach()
            {
                PushManager.Shared.PushActivatedEvent -= PushActivated;
            }

            //called when the event fires
            private void PushActivated(object sender, PushActivatedEventArgs e)
            {
                Logger.Info("Push activated!");

                UrbanAirship.Push.PushNotification notification = e.Notification;

                //only toasts can be activated
                if (notification.Type == UrbanAirship.Push.NotificationType.Toast)
                {
                    ToastNotificationData data = notification.ToastData;

                    Logger.Info("Text: ");
                    foreach (var item in data.Text)
                    {
                        Logger.Info(item);
                    }

                    //if needed, the full XML payload is included for further inspection
                    XmlDocument payload = (XmlDocument)data.Payload;
                    Logger.Info("Full XML payload:");
                    Logger.Info(payload.GetXml());
                }
            }
        }

        private RegistrationEventListener registrationListener;
        private PushReceivedEventListener pushReceivedListener;
        private PushActivatedEventListener pushActivatedListener;

        /// <summary>
        /// Initializes the singleton application object.  This is the first line of authored code
        /// executed, and as such is the logical equivalent of main() or WinMain().
        /// </summary>
        public App()
        {
            this.InitializeComponent();
            this.Suspending += OnSuspending;

            //initialize and register UA event listeners
            registrationListener = new RegistrationEventListener();
            pushReceivedListener = new PushReceivedEventListener();
            pushActivatedListener = new PushActivatedEventListener();
        }

        /// <summary>
        /// Invoked when the application is launched normally by the end user.  Other entry points
        /// will be used when the application is launched to open a specific file, to display
        /// search results, and so forth.
        /// </summary>
        /// <param name="args">Details about the launch request and process.</param>
        protected override void OnLaunched(LaunchActivatedEventArgs args)
        {

            Logger.Debug("Launched with: " + args.Arguments);

            Frame rootFrame = Window.Current.Content as Frame;

            // Do not repeat app initialization when the Window already has content,
            // just ensure that the window is active
            if (rootFrame == null)
            {
                // Create a Frame to act as the navigation context and navigate to the first page
                rootFrame = new Frame();

                if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                    //TODO: Load state from previously suspended application
                }

                // Place the frame in the current Window
                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                // When the navigation stack isn't restored navigate to the first page,
                // configuring the new page by passing required information as a navigation
                // parameter
                if (!rootFrame.Navigate(typeof(MainPage), args.Arguments))
                {
                    throw new Exception("Failed to create initial page");
                }
            }
            // Ensure the current window is active
            Window.Current.Activate();

            var config = AirshipConfig.ReadConfig("airshipconfig.xml");
            UAirship.TakeOff(config);
            config.DebugLog();

            //enable toasts
            PushManager.Shared.ToastEnabled = true;
        }

        /// <summary>
        /// Invoked when application execution is being suspended.  Application state is saved
        /// without knowing whether the application will be terminated or resumed with the contents
        /// of memory still intact.
        /// </summary>
        /// <param name="sender">The source of the suspend request.</param>
        /// <param name="e">Details about the suspend request.</param>
        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            //TODO: Save application state and stop any background activity
            deferral.Complete();
        }
    }
}
