﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using UrbanAirship;
using UrbanAirship.Push;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace Win8Sample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {

        /// <summary>
        /// Custom log writer that will allow us to log to both the console and the
        /// app UI.
        /// </summary>
        public sealed class CollectionLogWriter : ILogWriter
        {
            private Page page;

            public CollectionLogWriter(MainPage page)
            {
                this.LogTextObservableCollection = new ObservableCollection<string>();
                this.page = page;
            }

            public ObservableCollection<string> LogTextObservableCollection
            {
                get;
                private set;
            }

            public void WriteLine(String message)
            {
                // Log to console
                System.Diagnostics.Debug.WriteLine(message);

                // Update the UI on the UI thread
                page.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                {
                    LogTextObservableCollection.Add(message);
                }).AsTask();
            }
        }

        public CollectionLogWriter UILogWriter
        {
            get;
            set;
        }

        public ObservableCollection<string> Tags
        {
            get;
            private set;
        }

        public MainPage()
        {
            this.InitializeComponent();

            // Replace the original log writer with a custom
            // one that allows us to write to the screen
            this.UILogWriter = new CollectionLogWriter(this);
            Logger.LogWriter = this.UILogWriter;

            // Init the backing collection for the tags UI
            this.Tags = new ObservableCollection<string>(PushManager.Shared.Tags);

            // Register for Registration events.
            PushManager.Shared.RegistrationEvent += ApidRegistrationEvent; //TODO: remove event listener

            this.UILogWriter.LogTextObservableCollection.CollectionChanged += LogText_CollectionChanged; //TODO: remove event listener
        }

        void ApidRegistrationEvent(object sender, RegistrationEventArgs e)
        {
            // Update the UI on the UI thread
            this.ApidTextBox.Text = e.Apid;

            if (!e.IsValid)
            {
                this.ApidTextBox.Foreground = new SolidColorBrush(Colors.Red);
            }
            else
            {
                this.ApidTextBox.Foreground = new SolidColorBrush(Colors.Black);
            }
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.ToastsCheckbox.IsChecked = (PushManager.Shared.EnabledNotificationTypes & NotificationType.Toast) > 0;
            this.AliasTextBox.Text = PushManager.Shared.Alias;
            this.Tags = new ObservableCollection<string>(PushManager.Shared.Tags);
        }

        void LogText_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            this.LogMessageListBox.ScrollIntoView(this.UILogWriter.LogTextObservableCollection.Last());
        }

        private void ToastsEnabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.EnabledNotificationTypes = NotificationType.Toast;
            Logger.Debug("Toast Enabled. Enabled Types: " + (int)PushManager.Shared.EnabledNotificationTypes);
        }

        private void ToastsDisabled(object sender, RoutedEventArgs e)
        {
            PushManager.Shared.EnabledNotificationTypes = NotificationType.None;
            Logger.Debug("Toast Disabled. Enabled Types: " + (int)PushManager.Shared.EnabledNotificationTypes);
        }

        private void UpdateRegistrationButtonClick(object sender, RoutedEventArgs e)
        {
            // Update the APID with the attributes set in the UI
            // Set the new values first here
            PushManager.Shared.Alias = this.AliasTextBox.Text.Trim();
            PushManager.Shared.Tags = this.Tags;

            // Now update them on the server
            PushManager.Shared.UpdateRegistration();
        }

        private void AddTagClick(object sender, RoutedEventArgs e)
        {
            string newTag = this.TagEntryBox.Text.Trim();
            if (newTag.Length > 0 && !Tags.Contains(newTag))
            {
                this.TagEntryBox.Text = "";
                Tags.Add(newTag);
            }
        }

        private void RemoveTagClick(object sender, RoutedEventArgs e)
        {
            if (this.TagListBox.SelectedIndex >= 0)
            {
                this.Tags.RemoveAt(this.TagListBox.SelectedIndex);
            }
        }
    }
}
